import typing
from nubium_schemas import dc, pdc, AvroModel

from .schema_components import (
    _eloqua_hash_field,
    PersonalFacts,
    MarketingDescriptors,
    Privacy,
    LastSubmission,
    TrackingIds,
    Tombstone,
)


@pdc
class PersonSchema(AvroModel):
    personal_facts: PersonalFacts = dc.field(default_factory=PersonalFacts)
    marketing_descriptors: MarketingDescriptors = dc.field(default_factory=MarketingDescriptors)
    privacy: Privacy = dc.field(default_factory=Privacy)
    last_submission: LastSubmission = dc.field(default_factory=LastSubmission)
    tracking_ids: typing.Dict[str, typing.Dict[str, TrackingIds]] = dc.field(default_factory=lambda: {"domain": {"unique_id": TrackingIds().asdict()}})
    tombstone: Tombstone = dc.field(default_factory=Tombstone)
    last_evaluated_by_dwm: str = _eloqua_hash_field(default="")

    class Meta:
        schema_doc = False
        alias_nested_items = {
            "personal_facts": "PersonalFacts",
            "marketing_descriptors": "MarketingDescriptors",
            "privacy": "Privacy",
            "last_submission": "LastSubmission",
            "tracking_ids": "TrackingIds",
            "tombstone": "Tombstone",
        }


def person_empty_dict():
    return PersonSchema(**{"personal_facts": {"email_address": ""}}).asdict()


person_schema = PersonSchema.avro_schema_to_python()


@pdc
class SalesforceStreamEvent(AvroModel):
    event_replay_id: str = ""
    event_timestamp: str = ""
    event_type: str = ""
    salesforce_object_type: str = ""
    soql_query_fields: typing.Dict[str, str] = dc.field(default_factory=dict)

    class Meta:
        schema_doc = False


salesforce_stream_event = SalesforceStreamEvent.avro_schema_to_python()


def salesforce_stream_event_empty_dict():
    return SalesforceStreamEvent().asdict()



import typing
from nubium_schemas import dc, pdc, AvroModel


def _eloqua_hash_field(*args, **kwargs):
    kwargs["metadata"] = kwargs.get("metadata", {})
    kwargs["metadata"]["eloqua_hash"] = True
    return dc.field(*args, **kwargs)


@pdc
class Address(AvroModel):
    class Meta:
        schema_doc = False

    country_name: str = _eloqua_hash_field(default="")
    country_code: str = _eloqua_hash_field(default="")
    address_street_1: str = _eloqua_hash_field(default="")
    address_street_2: str = _eloqua_hash_field(default="")
    address_street_3: str = _eloqua_hash_field(default="")
    address_city: str = _eloqua_hash_field(default="")
    address_state_province: str = _eloqua_hash_field(default="")
    address_postal_code: str = _eloqua_hash_field(default="")
    core_based_statistical_area: str = _eloqua_hash_field(default="")
    combined_statistical_area: str = _eloqua_hash_field(default="")


@pdc
class Job(AvroModel):
    class Meta:
        schema_doc = False

    company: str = _eloqua_hash_field(default="")
    business_phone: str = _eloqua_hash_field(default="")
    fax_number: str = _eloqua_hash_field(default="")
    job_title: str = _eloqua_hash_field(default="")
    department: str = _eloqua_hash_field(default="")
    job_role: str = _eloqua_hash_field(default="")
    job_level: str = _eloqua_hash_field(default="")
    job_function: str = _eloqua_hash_field(default="")
    industry: str = _eloqua_hash_field(default="")
    annual_revenue: str = _eloqua_hash_field(default="")
    company_size: str = _eloqua_hash_field(default="")


@pdc
class PersonalFacts(AvroModel):
    class Meta:
        schema_doc = False
        alias_nested_items = {
            "address": "Address",
            "job": "Job",
        }

    email_address: str = _eloqua_hash_field()
    is_bounceback: str = _eloqua_hash_field(default="")
    is_a_test_contact: str = ""
    salutation: str = _eloqua_hash_field(default="")
    first_name: str = _eloqua_hash_field(default="")
    middle_name: str = _eloqua_hash_field(default="")
    last_name: str = _eloqua_hash_field(default="")
    mobile_phone: str = _eloqua_hash_field(default="")
    language_preference: str = _eloqua_hash_field(default="")
    address: Address = dc.field(default_factory=Address)
    job: Job = dc.field(default_factory=Job)


@pdc
class Mlsm(AvroModel):
    class Meta:
        schema_doc = False

    lead_ranking: str = _eloqua_hash_field(default="")
    lead_rating: str = _eloqua_hash_field(default="")
    interest_level: str = _eloqua_hash_field(default="")
    qualification_level: str = _eloqua_hash_field(default="")
    all_scores: str = _eloqua_hash_field(default="")


@pdc
class LeadScore(AvroModel):
    class Meta:
        schema_doc = False
        alias_nested_items = {"mlsm": "Mlsm"}

    mlsm: Mlsm = dc.field(default_factory=Mlsm)


@pdc
class MarketingDescriptors(AvroModel):
    class Meta:
        schema_doc = False
        alias_nested_items = {"lead_score": "LeadScore"}

    persona: str = _eloqua_hash_field(default="")
    super_region: str = _eloqua_hash_field(default="")
    sub_region: str = _eloqua_hash_field(default="")
    penalty_box_reason: str = ""
    penalty_box_expiration: str = _eloqua_hash_field(default="")
    lead_score: LeadScore = dc.field(default_factory=LeadScore)
    provisional_account_match: str = _eloqua_hash_field(default="")


@pdc
class Privacy(AvroModel):
    class Meta:
        schema_doc = False

    consent_email_marketing: str = _eloqua_hash_field(default="")
    consent_email_marketing_timestamp: str = ""
    consent_email_marketing_source: str = ""
    consent_share_to_partner: str = ""
    consent_share_to_partner_timestamp: str = ""
    consent_share_to_partner_source: str = ""
    consent_phone_marketing: str = ""
    consent_phone_marketing_timestamp: str = ""
    consent_phone_marketing_source: str = ""


@pdc
class OptIn(AvroModel):
    class Meta:
        schema_doc = False

    f_formdata_optin: str = ""
    f_formdata_optin_phone: str = ""
    f_formdata_sharetopartner: str = ""


@pdc
class Location(AvroModel):
    class Meta:
        schema_doc = False

    city_from_ip: str = ""
    state_province_from_ip: str = ""
    postal_code_from_ip: str = ""
    country_from_ip: str = _eloqua_hash_field(default="")
    country_from_dns: str = _eloqua_hash_field(default="")


@pdc
class CampaignResponseMetadata(AvroModel):
    class Meta:
        schema_doc = False

    ext_tactic_id: str = ""
    int_tactic_id: str = ""
    offer_id: str = ""
    offer_consumption_timestamp: str = ""
    is_lead_activity: str = ""


@pdc
class LastSubmission(AvroModel):
    class Meta:
        schema_doc = False
        alias_nested_items = {
            "opt_in": "OptIn",
            "location": "Location",
            "campaign_response_metadata": "CampaignResponseMetadata",
        }

    submission_date: str = ""
    submission_source: str = ""
    opt_in: OptIn = dc.field(default_factory=OptIn)
    location: Location = dc.field(default_factory=Location)
    campaign_response_metadata: CampaignResponseMetadata = dc.field(default_factory=CampaignResponseMetadata)


@pdc
class Tombstone(AvroModel):
    class Meta:
        schema_doc = False

    is_tombstoned: str = ""
    tombstone_timestamp: str = ""
    tombstone_source: str = ""
    delete_all_data: str = ""


@pdc
class TrackingIds(AvroModel):
    class Meta:
        schema_doc = False

    record_status: str = ""
    record_status_changed_date: str = ""
    created_date: str = ""
    attributes: typing.Dict[str, str] = dc.field(default_factory=dict)




bulk_data_chunk = {
  "name": "BulkDataChunk",
  "type": "array",
  "items": {
    "type": "map",
    "values": "string"},
  "default": []
}

bulk_data_record = {
  "name": "BulkDataRecord",
  "type": "record",
  "fields": [
    {"name": "record", "type": {"type": "map", "values": "string"}, "default": {}}
  ]
}


from os import environ


def cluster_topic_map():
    cluster_0 = environ.get('NUBIUM_CLUSTER_0', '')
    cluster_1 = environ.get('NUBIUM_CLUSTER_1', '')
    cluster_2 = environ.get('NUBIUM_CLUSTER_2', '')
    test_cluster = environ.get('TEST_CLUSTER', '')
    dict_out = {
        "CampaignResponse_CampaignResponseFromEloqua_InquiriesRetriever_Timestamps": cluster_0,
        "CampaignResponse_CampaignResponseFromEloqua_InquiriesToCanon": cluster_0,
        "CampaignResponse_Canon_Input": cluster_0,
        "CampaignResponse_Canon": cluster_0,
        "CampaignResponse_CampaignResponseToSalesforce_CampaignMembersToUpsert_FirstTry": cluster_0,
        "CampaignResponse_CampaignResponseToSalesforce_CampaignMembersToUpsert_Retry1": cluster_0,
        "CampaignResponse_CampaignResponseToSalesforce_CampaignMembersToUpsert_Retry2": cluster_0,
        "CampaignResponse_CampaignResponseToSalesforce_CampaignMembersToUpsert_Retry3": cluster_0,
        "CampaignResponse_CampaignResponseToSalesforce_CampaignMembersToUpsert_Failure": cluster_0,
        "NubiumIntegrations_Eloqua_CdoUpdates_FirstTry": cluster_0,
        "NubiumIntegrations_Eloqua_CdoUpdates_Retry1": cluster_0,
        "NubiumIntegrations_Eloqua_CdoUpdates_Retry2": cluster_0,
        "NubiumIntegrations_Eloqua_CdoUpdates_Retry3": cluster_0,
        "NubiumIntegrations_Eloqua_CdoUpdates_Failure": cluster_0,
        "PeopleStream_PeopleStreamFromEloqua_ContactRetriever_Timestamps": cluster_0,
        "PeopleStream_PeopleStreamFromEloqua_ContactToCanon": cluster_0,
        "PeopleStream_PeopleStreamFromEloqua_UnsubscribeRetriever_Timestamps": cluster_0,
        "PeopleStream_PeopleStreamFromEloqua_UnsubscribeToCanon": cluster_0,
        "NubiumIntegrations_Eloqua_EbbController": cluster_0,
        "NubiumIntegrations_Eloqua_EbbWorkerTasks": cluster_0,
        "NubiumIntegrations_Vivastream_ContactsVivastreamRetriever_Timestamps": cluster_0,
        "NubiumIntegrations_Vivastream_CdoToFormTransform": cluster_0,
        "NubiumIntegrations_UploadWizard_CdoToFormTransform": cluster_0,
        "NubiumIntegrations_UploadWizard_ContactsUploadsMembersRetriever_Timestamps": cluster_0,
        "NubiumIntegrations_RedHatTV_ContactsRedHatTVRetriever_Timestamps": cluster_0,
        "NubiumIntegrations_RedHatTV_CdoToFormTransform": cluster_0,
        "NubiumIntegrations_PartnerOutreach_ContactsActivitiesRetriever_Timestamps": cluster_0,

        "PeopleStream_Canon_Input": cluster_1,
        "PeopleStream_Canon": cluster_1,
        "PeopleStream_DataWashingMachine_ProcessedRecords": cluster_1,
        "PeopleStream_DataWashingMachine_Prewash": cluster_1,
        "PeopleStream_DataWashingMachine_DeptJobrolePersona": cluster_1,
        "PeopleStream_DataWashingMachine_AddressMsa": cluster_1,
        "PeopleStream_DataWashingMachine_Privacy": cluster_1,
        "NubiumIntegrations_Eloqua_ContactUpdates_FirstTry": cluster_1,
        "NubiumIntegrations_Eloqua_ContactUpdates_Retry1": cluster_1,
        "NubiumIntegrations_Eloqua_ContactUpdates_Retry2": cluster_1,
        "NubiumIntegrations_Eloqua_ContactUpdates_Retry3": cluster_1,
        "NubiumIntegrations_Eloqua_ContactUpdates_Failure": cluster_1,
        "NubiumIntegrations_PartnerOutreach_ProspectsToUpsert": cluster_1,
        "NubiumIntegrations_PartnerOutreach_CdoUpdater": cluster_1,
        "NubiumIntegrations_PartnerOutreach_CdoUpdater_Retry1": cluster_1,
        "NubiumIntegrations_PartnerOutreach_CdoUpdater_Retry2": cluster_1,
        "NubiumIntegrations_PartnerOutreach_CdoUpdater_Failure": cluster_1,

        # TODO c1 PS3 - to sf filter and lead/contact updaters

        "NubiumIntegrations_Eloqua_FormPoster_FromDyFo_FirstTry": cluster_2,
        "NubiumIntegrations_Eloqua_FormPoster_FirstTry": cluster_2,
        "NubiumIntegrations_Eloqua_FormPoster_Retry1": cluster_2,
        "NubiumIntegrations_Eloqua_FormPoster_Retry2": cluster_2,
        "NubiumIntegrations_Eloqua_FormPoster_Retry3": cluster_2,
        "NubiumIntegrations_Eloqua_FormPoster_Failure": cluster_2,
        "NubiumIntegrations_Partner_BulkReceiver_Chunks": cluster_2,
        "NubiumIntegrations_Partner_BulkProcessor_Records": cluster_2,
        "NubiumIntegrations_DynamicForm_FormSubmissions": cluster_2,
        "NubiumIntegrations_DynamicForm_SpamFilter_CheckEmailAddress": cluster_2,
        "NubiumIntegrations_DynamicForm_SpamFilter_CheckVerifyId": cluster_2,
        "NubiumIntegrations_DynamicForm_SpamFilter_CheckSubmitId": cluster_2,
        "NubiumIntegrations_DynamicForm_SpamPosts": cluster_2,
        "NubiumIntegrations_DynamicForm_ErrorPosts": cluster_2,
        "PathFactory_PathFactory_DuplicatesFilter": cluster_2,
        "PathFactory_PathFactory_DuplicateClosedSessions": cluster_2,
        "PathFactory_PathFactory_ClosedSessions_FirstTry": cluster_2,
        "PathFactory_PathFactory_ClosedSessions_Retry1": cluster_2,
        "PathFactory_PathFactory_ClosedSessions_Retry2": cluster_2,
        "PathFactory_PathFactory_ClosedSessions_Failure": cluster_2,
        "NubiumIntegrations_PartnerOutreach_Cdo2ProspectTransform": cluster_2,
        # TODO c2 PS3 - from sf lead/contact retrievers
    }
    if test_cluster:
        dict_out.update({f'{k}__TEST': test_cluster for k in dict_out})
    return dict_out



from .schemas import PersonSchema


def _generate_eloqua_person_mask(cls):
    return {
        field.name: field.metadata.get(
            "eloqua_hash", _generate_eloqua_person_mask(field.type) if hasattr(field.type, "metadata") else False
        )
        for field in cls.get_fields()
    }


eloqua_person_mask = _generate_eloqua_person_mask(PersonSchema)



import typing
from nubium_schemas import dc, pdc, AvroModel
from .schema_components import CampaignResponse, TrackingIds, SalesforceCampaignMember


@pdc
class Canon(AvroModel):
    class Meta:
        schema_doc = False
        alias_nested_items = {
            "campaign_response": "CampaignResponse",
            "tracking_ids": "TrackingIds",
        }

    campaign_response: CampaignResponse = dc.field(default_factory=CampaignResponse)
    tracking_ids: TrackingIds = dc.field(default_factory=TrackingIds)
    raw_formdata: typing.Dict[str, str] = dc.field(default_factory=dict)


canon = Canon.avro_schema_to_python()

@pdc
class CampaignMembersCreate(AvroModel):
    class Meta:
        schema_doc = False
        alias_nested_items = {
            "campaign_response": "CampaignResponse",
            "tracking_ids": "TrackingIds",
        }

    campaign_response: CampaignResponse = dc.field(default_factory=CampaignResponse)
    tracking_ids: TrackingIds = dc.field(default_factory=TrackingIds)
    campaign_members_to_create: typing.List[typing.Union[SalesforceCampaignMember, None]] = dc.field(default_factory=lambda: [SalesforceCampaignMember().asdict()])


campaign_members_create = CampaignMembersCreate.avro_schema_to_python()

@pdc
class CampaignMembersUpdate(AvroModel):
    class Meta:
        schema_doc = False

    campaign_members_to_update: typing.List[typing.Union[SalesforceCampaignMember, None]] = dc.field(default_factory=lambda: [SalesforceCampaignMember().asdict()])


campaign_members_update = CampaignMembersUpdate.avro_schema_to_python()



# TODO: Remove NI imports once apps are updated to reference the "correct"
#  schemas since they are here just for backwards compatibility during app product transfer
from .schemas import (
    eloqua_cdo_record_data,
    eloqua_cdo_record_update_data,
    eloqua_contact_record_data,
    eloqua_retriever_timestamp)



from nubium_schemas.nubium_integrations.eloqua import (
    eloqua_cdo_record_data,
    eloqua_cdo_record_update_data,
    eloqua_contact_record_data)

eloqua_retriever_timestamp = {
    "name": "EloquaRetrieverTimestamp",
    "type": "record",
    "fields": [
        {"name": "timestamp", "type": "string", "default": ""}
    ]
}



from nubium_schemas import dc, pdc, AvroModel

from .schema_components import (
    EloquaContactRecord,
    ebb_job_init_fields,
    ebb_job_eloqua_metadata,
    ebb_worker_task_fields,
    eloqua_cdo_record,
    eloqua_cdo_record_unique_field)

ebb = {
    "name": "EloquaBulkBatcher",
    "type": "record",
    "fields": [
        {"name": "controller_job_id", "type": "string", "default": ""},
        {"name": "controller_job_status_update", "type": "string", "default": ""},
        {"name": "job_init_fields", "type": ebb_job_init_fields},
        {"name": "job_eloqua_metadata", "type": ebb_job_eloqua_metadata},
        {"name": "worker_task_fields", "type": ebb_worker_task_fields}
    ]
}

eloqua_cdo_record_data = {
    "name": "EloquaCdoRecordData",
    "type": "record",
    "fields": [
        {"name": "eloqua_cdo_record", "type": eloqua_cdo_record}
    ]
}

eloqua_cdo_record_update_data = {
    "name": "EloquaCdoRecordUpdateData",
    "type": "record",
    "fields": [
        {"name": "eloqua_cdo_record", "type": eloqua_cdo_record},
        {"name": "eloqua_cdo_record_unique_field", "type": eloqua_cdo_record_unique_field}
    ]
}


@pdc
class EloquaContactRecordData(AvroModel):
    eloqua_contact_record: EloquaContactRecord = dc.field(default_factory=EloquaContactRecord)

    class Meta:
        schema_doc = False
        alias_nested_items = {
            "eloqua_contact_record": "EloquaContactRecord"
        }


def eloqua_contact_record_data_empty_dict():
    return EloquaContactRecordData(**{}).asdict()


eloqua_contact_record_data = EloquaContactRecordData.avro_schema_to_python()


eloqua_form = {
    "name": "EloquaForm",
    "type": "record",
    "fields": [
        {"name": "form_data", "type": {"type": "map", "values": "string"}, "default": {}}
    ]
}

post_eloqua_form = {
    "name": "PostEloquaForm",
    "type": "record",
    "fields": [
        {"name": "url", "type": "string", "doc": "Eloqua form endpoint URL", "default": ""},
        {"name": "form_data", "type": {"type": "map", "values": "string"}, "default": {}}
    ]
}


import typing

from nubium_schemas import dc, pdc, AvroModel

ebb_job_init_fields = {
    "name": "EbbJobInitFields",
    "type": "record",
    "fields": [
        {"name": "eloqua_source_name", "type": "string", "default": ""},
        {"name": "eloqua_fields_out", "type": "string", "default": ""},
        {"name": "eloqua_field_filters", "type": "string", "default": ""},
        {"name": "from_timestamp", "type": "string", "default": ""},
        {"name": "to_timestamp", "type": "string", "default": ""},
        {"name": "nubium_created_by_app_name", "type": "string", "default": ""},
        {"name": "nubium_bulk_output_topic", "type": "string", "default": ""},
        {"name": "nubium_rest_timestamp_topic", "type": "string", "default": ""}
    ]
}

ebb_job_eloqua_metadata = {
    "name": "EbbJobEloquaMetadata",
    "type": "record",
    "fields": [
        {"name": "bulk_export_definition", "type": "string", "default": ""},
        {"name": "bulk_sync_uri", "type": "string", "default": ""},
        {"name": "bulk_total_records", "type": "string", "default": ""}
    ]
}

ebb_worker_task_fields = {
    "name": "EbbWorkerTaskFields",
    "type": "record",
    "fields": [
        {"name": "task_command", "type": "string", "default": ""},
        {"name": "task_attempt", "type": "string", "default": ""},
        {"name": "bulk_offset", "type": "string", "default": ""},
        {"name": "bulk_batch_size", "type": "string", "default": ""}
    ]
}

eloqua_cdo_record = {
    "name": "EloquaCdoRecord",
    "type": "record",
    "fields": [
        {"name": "cdo_id", "type": "string", "default": ""},
        {"name": "cdo_record_id", "type": "string", "default": ""},
        {"name": "field_map", "type": {"type": "map", "values": "string"}, "default": {}}
    ]
}

eloqua_cdo_record_unique_field = {
    "name": "EloquaCdoRecordUniqueField",
    "type": "record",
    "fields": [
        {"name": "cdo_unique_field", "type": "string", "default": ""},
        {"name": "record_unique_field_value", "type": "string", "default": ""}
    ]
}


@pdc
class EloquaContactRecord(AvroModel):
    class Meta:
        schema_doc = False

    contact_id: str = ""
    field_map: typing.Dict[str, str] = dc.field(default_factory=dict)



from copy import deepcopy

def hrs_to_ms(hours):
    return 1000 * 60 * 60 * hours

def mb_to_bytes(mb):
    return 1024 * 1024 * mb

config_default = {
    "cleanup.policy": "delete",
    "retention.ms": hrs_to_ms(72),
    "segment.ms": hrs_to_ms(72),
    "segment.bytes": mb_to_bytes(100)
}

config_changelog = {
  **config_default,
  "cleanup.policy": "compact"
}

class TopicConfig:
    pass

class PrimaryTopicConfig(TopicConfig):
    def __init__(self, parts, rep_factor=3, config=None):
        self.partitions = parts
        self.replication_factor = rep_factor
        if not config:
          config = deepcopy(config_default)
        self.config = config


class InternalTopicConfig(TopicConfig):
    def __init__(self, config=None):
        if not config:
          config = deepcopy(config_default)
        self.config = config


class ChangelogTopicConfig(TopicConfig):
    def __init__(self, config=None):
        if not config:
          config = deepcopy(config_changelog)
        self.config = config


primary_topic_configs = {
    "PeopleStream": {
        "Canon": [
            PrimaryTopicConfig(parts=30),
            {"Input": PrimaryTopicConfig(parts=30)},
        ],
        "PeopleStreamFromEloqua": {
            "ContactRetriever": {"Timestamps": PrimaryTopicConfig(parts=1)},
            "ContactToCanon": PrimaryTopicConfig(parts=15),
            "UnsubscribeRetriever": {"Timestamps": PrimaryTopicConfig(parts=1)},
            "UnsubscribeToCanon": PrimaryTopicConfig(parts=3),
        },
        "DataWashingMachine": {
            "ProcessedRecords": PrimaryTopicConfig(parts=15),
            "Prewash": PrimaryTopicConfig(parts=15),
            "DeptJobrolePersona": PrimaryTopicConfig(parts=15),
            "AddressMsa": PrimaryTopicConfig(parts=15),
            "Privacy": PrimaryTopicConfig(parts=15),
        },
    },

    "CampaignResponse": {
        "Canon": [
            PrimaryTopicConfig(parts=9),
            {"Input": PrimaryTopicConfig(parts=9)},
        ],
        "CampaignResponseFromEloqua": {
            "InquiriesRetriever": {"Timestamps": PrimaryTopicConfig(parts=1)},
            "InquiriesToCanon": PrimaryTopicConfig(parts=6),
        },
        "CampaignResponseToSalesforce": {
            "CampaignMembersToUpsert": {
                "FirstTry": PrimaryTopicConfig(parts=9),
                "Retry1": PrimaryTopicConfig(parts=3),
                "Retry2": PrimaryTopicConfig(parts=1),
                "Retry3": PrimaryTopicConfig(parts=1),
                "Failure": PrimaryTopicConfig(parts=1),
            },
        },
    },

    "NubiumIntegrations": {
        "DynamicForm": {
            "FormSubmissions": PrimaryTopicConfig(parts=6),
            "SpamFilter": {
                "CheckEmailAddress": PrimaryTopicConfig(parts=6),
                "CheckVerifyId": PrimaryTopicConfig(parts=6),
                "CheckSubmitId": PrimaryTopicConfig(parts=6),
            },
            "SpamPosts": PrimaryTopicConfig(parts=3),
            "ErrorPosts": PrimaryTopicConfig(parts=1)
        },
        "Eloqua": {
            "EbbController": PrimaryTopicConfig(parts=3),
            "EbbWorkerTasks": PrimaryTopicConfig(parts=15),
            "FormPoster": {
                "FromDyFo": {
                    "FirstTry": PrimaryTopicConfig(parts=9)},
                "FirstTry": PrimaryTopicConfig(parts=9),
                "Retry1": PrimaryTopicConfig(parts=1),
                "Retry2": PrimaryTopicConfig(parts=1),
                "Retry3": PrimaryTopicConfig(parts=1),
                "Failure": PrimaryTopicConfig(parts=1),
            },
            "CdoUpdates": {
                "FirstTry": PrimaryTopicConfig(parts=9),
                "Retry1": PrimaryTopicConfig(parts=1),
                "Retry2": PrimaryTopicConfig(parts=1),
                "Retry3": PrimaryTopicConfig(parts=1),
                "Failure": PrimaryTopicConfig(parts=1),
            },
            "ContactUpdates": {
                "FirstTry": PrimaryTopicConfig(parts=30),
                "Retry1": PrimaryTopicConfig(parts=1),
                "Retry2": PrimaryTopicConfig(parts=1),
                "Retry3": PrimaryTopicConfig(parts=1),
                "Failure": PrimaryTopicConfig(parts=1),
            },
        },
        "Partner": {
            "BulkReceiver": {"Chunks": PrimaryTopicConfig(parts=1)},
            "BulkProcessor": {"Records": PrimaryTopicConfig(parts=6)},
        },
        "Vivastream": {
            "ContactsVivastreamRetriever": {"Timestamps": PrimaryTopicConfig(parts=1)},
            "CdoToFormTransform": PrimaryTopicConfig(parts=6),
        },
        "UploadWizard": {
            "ContactsUploadsMembersRetriever": {"Timestamps": PrimaryTopicConfig(parts=1)},
            "CdoToFormTransform": PrimaryTopicConfig(parts=6),
        },
        "RedHatTV": {
            "ContactsUploadsMembersRetriever": {"Timestamps": PrimaryTopicConfig(parts=1)},
            "CdoToFormTransform": PrimaryTopicConfig(parts=6),
        },
    },

    "PathFactory": {
        "PathFactory": {
            "DuplicatesFilter": PrimaryTopicConfig(parts=6),
            "DuplicateClosedSessions": PrimaryTopicConfig(parts=1),
            "ClosedSessions": {
                "FirstTry": PrimaryTopicConfig(parts=6),
                "Retry1": PrimaryTopicConfig(parts=1),
                "Retry2": PrimaryTopicConfig(parts=1),
                "Failure": PrimaryTopicConfig(parts=1),
            },
        },
    },
}

internal_topic_configs = {
    "PeopleStream": {
        "DataWashingMachine": {
            "Prewash": {"Internal_PeopleStream_DataWashingMachine_Prewasher": InternalTopicConfig()},
            "DeptJobrolePersona": {"Internal_PeopleStream_DataWashingMachine_DeptJobrolePersonaDeriver": InternalTopicConfig()},
            "AddressMsa": {"Internal_PeopleStream_DataWashingMachine_AddressMsa": InternalTopicConfig()},
            "ProcessedRecords": {"Internal_PeopleStream_PeopleStreamCanon_Aggregator": InternalTopicConfig()},
        },
        "Canon": {
            "Input": {"Internal_PeopleStream_PeopleStreamCanon_Aggregator": InternalTopicConfig()},
            "Internal_PeopleStream_PeopleStreamToEloqua_Filter": InternalTopicConfig(),
        },
    },

    "CampaignResponse": {
        "Canon": {
            "Input": {"Internal_CampaignResponse_CampaignResponseCanon_Aggregator": InternalTopicConfig()},
            "Internal_CampaignResponse_CampaignResponseToEloqua_Filter": InternalTopicConfig(),
            "Internal_CampaignResponse_CampaignResponseToSalesforce_Filter": InternalTopicConfig(),
        },
    },

    "NubiumIntegrations": {
        "DynamicForm": {
            "SpamFilter": {
                "CheckEmailAddress-GroupBy-SpamKeyedRecord.submission_id": InternalTopicConfig(),
                "CheckSubmitId-GroupBy-SpamKeyedRecord.submission_id": InternalTopicConfig(),
                "CheckVerifyId-GroupBy-SpamKeyedRecord.verification_id": InternalTopicConfig(),
            },
        },
        "Eloqua": {
            "EbbController": {
                "Internal_NubiumIntegrations_Eloqua_EbbController": InternalTopicConfig(),
                "Internal_NubiumIntegrations_Eloqua_EbbController-GroupBy-job_id": InternalTopicConfig(),
            },
        },
    },

    "PathFactory": {
        "Pathfactory": {
            "DuplicatesFilter": {
                "Internal_PathFactory_PathFactory_DuplicatesFilter": InternalTopicConfig(),
                "Internal_PathFactory_PathFactory_DuplicatesFilter-GroupBy-session_id_check": InternalTopicConfig(),
            },
        },
    },
}

internal_changelog_configs = {
    "PeopleStream": {
        "PeopleStreamCanon": {"Aggregator-people-stream-canon-changelog": ChangelogTopicConfig()},
        "DataWashingMachine": {"Prewasher-dwm_scrub_table-changelog": ChangelogTopicConfig()},
    },
    "CampaignResponse": {
        "CampaignResponseCanon": {"Aggregator-canon-changelog": ChangelogTopicConfig()}
    },

    "NubiumIntegrations": {
        "DynamicForm": {
            "SpamFilter-submit_table-changelog": ChangelogTopicConfig(),
            "SpamFilter-verify_table-changelog": ChangelogTopicConfig(),
        },
        "Eloqua": {
            "EbbController-bulk_job_data_table-changelog": ChangelogTopicConfig(),
            "EbbController-bulk_job_tasks_table-changelog": ChangelogTopicConfig(),
        },
    },

    "PathFactory": {
        "PathFactory": {"DuplicatesFilter-session_id_table-changelog": ChangelogTopicConfig()},
    },
}


def unpack_topic_dict(topic_dict, prev_layer=''):
    if isinstance(topic_dict, dict):
        return {
            name: config
            for key, value in topic_dict.items()
            for name, config in unpack_topic_dict(value, f'{prev_layer}_{key}' if prev_layer else key).items()
        }
    elif isinstance(topic_dict, list):
        return {
            name: config
            for item in topic_dict
            for name, config in unpack_topic_dict(item, prev_layer).items()
        }
    elif isinstance(topic_dict, TopicConfig):
       return {prev_layer: topic_dict}
    raise ValueError(f"unsupported type for topic_dict: {type(topic_dict)}")


primary_topics = unpack_topic_dict(primary_topic_configs)
internal_topics = unpack_topic_dict(internal_topic_configs)
changelog_topics = unpack_topic_dict(internal_changelog_configs)


from nubium_schemas import dc, pdc, AvroModel
from .schema_components import ProspectFields


@pdc
class OutreachProspectToUpsert(AvroModel):

    class Meta:
        schema_doc = False

    attributes: ProspectFields = dc.field(default_factory=ProspectFields)
    prospectID: str = ""
    eloqua_contact_id: str = ""
    is_subscribed: str = ""
    cdo_id: str = ""
    cdo_record_id: str = ""


def outreach_prospect_to_upsert_empty_dict():
    return OutreachProspectToUpsert(**{}).asdict()


outreach_prospect_to_upsert = OutreachProspectToUpsert.avro_schema_to_python()



from nubium_schemas import dc, pdc, AvroModel

from .schema_components import SalesforceRecord


@pdc
class SalesforceRecordData(AvroModel):
    salesforce_record_data: SalesforceRecord = dc.field(default_factory=SalesforceRecord)

    class Meta:
        schema_doc = False
        alias_nested_items = {
            "salesforce_record": "SalesforceRecord"
        }


def salesforce_record_data_empty_dict():
    return SalesforceRecordData(**{}).asdict()


salesforce_record_data = SalesforceRecordData.avro_schema_to_python()



from nubium_schemas.campaign_response import Canon
from nubium_schemas.campaign_response.schema_components import CampaignResponse, TrackingIds 


def test_data_models_can_generate_default_dictionaries():
    expected = {
        "campaign_response": {
            "email_address": "",
            "ext_tactic_id": "",
            "int_tactic_id": "",
            "offer_consumption_timestamp": "",
            "offer_id": "",
        },
        "raw_formdata": {},
        "tracking_ids": {
            "eloqua_contacts_inquiries_id": "",
            "sfdc_contact_id": "",
            "sfdc_ext_tactic_contact_id": "",
            "sfdc_ext_tactic_lead_id": "",
            "sfdc_int_tactic_contact_id": "",
            "sfdc_int_tactic_lead_id": "",
            "sfdc_lead_id": "",
            "sfdc_offer_contact_id": "",
            "sfdc_offer_lead_id": "",
        },
    }
    assert Canon().asdict() == expected


def test_data_models_can_generate_avro_schema_in_python_dict_form():
    old_canon = {
        "name": "Canon",
        "type": "record",
        "fields": [
            {"name": "campaign_response", "type": CampaignResponse.avro_schema_to_python()},
            {"name": "tracking_ids", "type": TrackingIds.avro_schema_to_python()},
            {"name": "raw_formdata", "type": {"name": "raw_formdatum", "type": "map", "values": "string"}, "default": {}}
         ]
    }
    assert Canon.avro_schema_to_python() == old_canon



from .schemas import (
    ebb,
    eloqua_cdo_record_data,
    eloqua_cdo_record_update_data,
    eloqua_cdo_record_unique_field,
    eloqua_contact_record_data,
    eloqua_form,
    post_eloqua_form)



from nubium_schemas.people_stream.hash_fields import eloqua_person_mask


expected_eloqua_person_mask = {
    "last_evaluated_by_dwm": True,
    "last_submission": {
        "campaign_response_metadata": {
            "ext_tactic_id": False,
            "int_tactic_id": False,
            "is_lead_activity": False,
            "offer_consumption_timestamp": False,
            "offer_id": False,
        },
        "location": {
            "city_from_ip": False,
            "country_from_dns": True,
            "country_from_ip": True,
            "postal_code_from_ip": False,
            "state_province_from_ip": False,
        },
        "opt_in": {"f_formdata_optin": False, "f_formdata_optin_phone": False, "f_formdata_sharetopartner": False},
        "submission_date": False,
        "submission_source": False,
    },
    "marketing_descriptors": {
        "lead_score": {
            "mlsm": {
                "all_scores": True,
                "interest_level": True,
                "lead_ranking": True,
                "lead_rating": True,
                "qualification_level": True,
            }
        },
        "penalty_box_expiration": True,
        "penalty_box_reason": False,
        "persona": True,
        "sub_region": True,
        "super_region": True,
        "provisional_account_match": True,
    },
    "personal_facts": {
        "address": {
            "address_city": True,
            "address_postal_code": True,
            "address_state_province": True,
            "address_street_1": True,
            "address_street_2": True,
            "address_street_3": True,
            "combined_statistical_area": True,
            "core_based_statistical_area": True,
            "country_code": True,
            "country_name": True,
        },
        "email_address": True,
        "first_name": True,
        "is_a_test_contact": False,
        "is_bounceback": True,
        "job": {
            "annual_revenue": True,
            "business_phone": True,
            "company": True,
            "company_size": True,
            "department": True,
            "fax_number": True,
            "industry": True,
            "job_function": True,
            "job_level": True,
            "job_role": True,
            "job_title": True,
        },
        "language_preference": True,
        "last_name": True,
        "middle_name": True,
        "mobile_phone": True,
        "salutation": True,
    },
    "privacy": {
        "consent_email_marketing": True,
        "consent_email_marketing_source": False,
        "consent_email_marketing_timestamp": False,
        "consent_phone_marketing": False,
        "consent_phone_marketing_source": False,
        "consent_phone_marketing_timestamp": False,
        "consent_share_to_partner": False,
        "consent_share_to_partner_source": False,
        "consent_share_to_partner_timestamp": False,
    },
    "tombstone": {
        "delete_all_data": False,
        "is_tombstoned": False,
        "tombstone_source": False,
        "tombstone_timestamp": False,
    },
    "tracking_ids": False,
}


def test_bool_hash_dict_can_be_generated_from_schema_classes():
    assert eloqua_person_mask == expected_eloqua_person_mask



import typing

from nubium_schemas import dc, pdc, AvroModel


@pdc
class SalesforceRecord(AvroModel):
    class Meta:
        schema_doc = False

    id: str = ""
    field_map: typing.Dict[str, str] = dc.field(default_factory=dict)



# argv[1] is the '.' delimited python path to a (schema) module.
# argv[2] is the filepath to dump to.
from sys import argv
import json
import importlib

module, schema = argv[1].rsplit('.', 1)
module = importlib.import_module(module)
schema = getattr(module, schema)

with open(argv[2], 'w') as f:
    json.dump(schema, f)



def empty_schema_field_dict(schema_dict):
    """Return the schema fields as a typical dict (that you would normally interact with) with all fields empty
    NOTE: Array types will return an array with ONE instance of the the expected object type"""
    if schema_dict.get('fields'):
        return {field['name']: '' if isinstance(field['type'], str) else empty_schema_field_dict(field['type'])
                for field in schema_dict['fields']}
    return [empty_schema_field_dict(schema_dict['items'])]


import setuptools

with open("README.md", "r") as file_obj:
    long_description = file_obj.read()

packages = setuptools.find_packages()

setuptools.setup(
    name='nubium-schemas',
    author="Edward Brennan",
    author_email="ebrennan@redhat.com",
    description="Python dictionary representations of Avro Schema for the nubium project",
    maintainer="Pramod Toraskar",
    maintainer_email="ptoraska@redhat.com",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://gitlab.corp.redhat.com/mkt-ops-de/nubium-schemas.git",
    packages=packages,
    install_requires=[
        "dataclasses-avroschema>=0.20.2",
        "pydantic"
    ],
    extras_require={
        "dev": ["black", "pip-tools", "pytest", "pytest-cov", "setuptools_scm"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from nubium_schemas import pdc, AvroModel

@pdc
class CampaignResponse(AvroModel):
    class Meta:
        schema_doc = False

    email_address: str = ""
    ext_tactic_id: str = ""
    int_tactic_id: str = ""
    offer_id: str = ""
    offer_consumption_timestamp: str = ""


@pdc
class TrackingIds(AvroModel):
    class Meta:
        schema_doc = False

    eloqua_contacts_inquiries_id: str = ""
    sfdc_contact_id: str = ""
    sfdc_lead_id: str = ""
    sfdc_ext_tactic_lead_id: str = ""
    sfdc_int_tactic_lead_id: str = ""
    sfdc_offer_lead_id: str = ""
    sfdc_ext_tactic_contact_id: str = ""
    sfdc_int_tactic_contact_id: str = ""
    sfdc_offer_contact_id: str = ""


@pdc
class SalesforceCampaignMember(AvroModel):
    class Meta:
        schema_doc = False

    campaign_membership_id: str = ""
    campaign_id: str = ""
    related_campaign_id: str = ""
    sfdc_contact_id: str = ""
    sfdc_lead_id: str = ""
    true_response_date: str = ""
    status: str = ""
    campaign_member_type: str = ""



import typing
from nubium_schemas import dc, pdc, AvroModel

@pdc
class ProspectFields(AvroModel):
    class Meta:
        schema_doc = False

    firstName: str = ""
    lastName: str = ""
    emails: typing.List[str] = dc.field(default_factory=list)
    workPhones: typing.List[str] = dc.field(default_factory=list)
    mobilePhones: typing.List[str] = dc.field(default_factory=list)
    company: str = ""
    occupation: str = ""
    addressStreet: str = ""
    addressStreet2: str = ""
    addressCity: str = ""
    addressState: str = ""
    addressZip: str = ""
    addressCountry: str = ""